@@ -7,9 +7,13 @@
  *
  */
 
+#include "config.h"
+
 /* regular GNU system includes */
 #include <stdio.h>
-#include <stdlib.h>
+#ifdef HAVE_STDLIB_H
+  #include <stdlib.h>
+#endif
 #include <math.h>
 #include <time.h>
 
@@ -26,13 +30,16 @@
 #include "out.xpm"
 
 /* called when caller id monitor state check button is toggled */
-void cid_toggle_cb(GtkWidget *widget, gpointer data, guint action) {
+void cid_toggle_cb(GtkWidget *widget _U_, gpointer data, guint action _U_) {
   struct session_t *session = (struct session_t *) data;
 
-  if (GTK_CHECK_MENU_ITEM (session->cid_check_menu_item)->active)
+  if (GTK_CHECK_MENU_ITEM (session->cid_check_menu_item)->active) {
     gtk_widget_show(session->cid);
-  else
+    session->option_show_callerid = 1;
+  } else {
     gtk_widget_hide(session->cid);
+    session->option_show_callerid = 0;
+  }
 }
 
 /*
