@@ -7,10 +7,14 @@
  *
  */
 
+#include "config.h"
+
 /* regular GNU system includes */
 #include <stdio.h>
 #include <signal.h>
-#include <stdlib.h>
+#ifdef HAVE_STDLIB_H
+  #include <stdlib.h>
+#endif
 #include <string.h>
 #include <time.h>
 
@@ -25,6 +29,8 @@
 #include "sound.h"
 #include "callerid.h"
 #include "llcheck.h"
+#include "settings.h"
+#include "gtksettings.h"
 
 /* graphical symbols */
 #include "hangup.xpm"
@@ -43,11 +49,15 @@ volatile int interrupted = 0; /* the caught signal will be stored here */
  *
  * input: data: session
  */
-static void quit(GtkWidget *widget, gpointer data, guint action) {
+static void quit(GtkWidget *widget _U_, gpointer data, guint action _U_) {
   struct session_t *session = (struct session_t *) data;
-
+  
+  settings_history_write(session); /* write history */
+  
   gtk_handle_hang_up_button(NULL, data); /* simulate hang_up_button */
 
+  session_set_state(session, STATE_SERVICE);
+
   /* some (GUI) de-initialization, not directly session related */
   llcheck_bar_deinit(session->llcheck_in);
   llcheck_bar_deinit(session->llcheck_out);
@@ -61,7 +71,7 @@ static void quit(GtkWidget *widget, gpointer data, guint action) {
  * input: data: session
  */
 gint delete_event(GtkWidget *widget,
-		  GdkEvent *event,
+		  GdkEvent *event _U_,
 		  gpointer data)
 {
   quit(widget, data, 0);
@@ -167,95 +177,25 @@ GtkWidget *ok_dialog_get(char *title, char *contents,
   return window;
 }
 
+/*
+ * shortcut to display a note about audio devices not available
+ */
+void show_audio_error_dialog() {
+  GtkWidget *dialog;
+
+  dialog = ok_dialog_get("ANT Note",
+			 "Can't open audio device.\n"
+			 "Please stop other applications using\n"
+			 "the audio device(s) and try again.",
+			 GTK_JUSTIFY_CENTER);
+  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
+  gtk_widget_show(dialog);
+}
+
 /*
  * some GUI callbacks
  */
 
-/* clicked "ok" at cb_settings */
-static void cb_settings_ok(GtkWidget *widget, gpointer data) {
-  struct session_t *session = (struct session_t *) data;
-  GtkWidget *window;
-  GtkWidget *entry;
-
-  /* initialization is just for correctness to make safe state */
-  char *msn = DEFAULT_MSN;
-  char *msns = DEFAULT_MSNS;
-  char *audio_device_name_in = DEFAULT_AUDIO_DEVICE_NAME_IN;
-  char *audio_device_name_out = DEFAULT_AUDIO_DEVICE_NAME_OUT;
-
-  /* save current settings */
-  char *old_msn = strdup(session->msn);
-  char *old_msns = strdup(session->msns);
-  char *old_audio_device_name_in = strdup(session->audio_device_name_in);
-  char *old_audio_device_name_out = strdup(session->audio_device_name_out);
-
-  int successful = 1;
-  GtkWidget *ok_window;
-
-  /* msn */
-  entry = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(widget), "msn_entry");
-  if (entry)
-    msn = gtk_entry_get_text(GTK_ENTRY(entry));
-  else
-    fprintf(stderr, "cb_settings_ok: Error getting msn.\n");
-
-  /* msns */
-  entry = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(widget), "msns_entry");
-  if (entry)
-    msns = gtk_entry_get_text(GTK_ENTRY(entry));
-  else
-    fprintf(stderr, "cb_settings_ok: Error getting msns.\n");
-       
-
-  /* audio_device_name_in */
-  entry = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(widget),
-					    "audio_device_name_in_entry");
-  if (entry)
-    audio_device_name_in = gtk_entry_get_text(GTK_ENTRY(entry));
-  else
-    fprintf(stderr, "cb_settings_ok: Error getting audio_device_name_in.\n");
-
-  /* audio_device_name_out */
-  entry = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(widget),
-					    "audio_device_name_out_entry");
-  if (entry)
-    audio_device_name_out = gtk_entry_get_text(GTK_ENTRY(entry));
-  else
-    fprintf(stderr, "cb_settings_ok: Error getting audio_device_name_out.\n");
-				   
-  /* reset all this by closing and reopening isdn and sound devices */
-  session_io_handlers_stop(session);
-  session_deinit(session);
-  if (session_init(session, audio_device_name_in, audio_device_name_out,
-		   msn, msns)) { /* new settings are bad */
-    /* restore and hope for the best */
-    session_init(session, old_audio_device_name_in, old_audio_device_name_out,
-		 old_msn, old_msns);
-    successful = 0;
-  }
-  session_io_handlers_start(session);
-
-  if (successful) { /* changed settings */
-    /* destroy the window (if possible - should be! ;)) */
-    window = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(widget),
-					       "dialog_window");
-    if (window) {
-      gtk_widget_destroy(window);
-    }
-  } else { /* settings error -> restored old settings -> display note */
-    ok_window = ok_dialog_get("Note",
-			      "Bad option, please try again.",
-			      GTK_JUSTIFY_CENTER);
-    gtk_window_set_modal(GTK_WINDOW(ok_window), TRUE);
-    gtk_widget_show(ok_window);
-  }
-
-  free(old_msn);
-  free(old_msns);
-  free(old_audio_device_name_in);
-  free(old_audio_device_name_out);
-}
-
 /*
  * File Menu entries
  */
@@ -266,18 +206,29 @@ struct info_row_t {
 };
 
 /* Info window */
-static void cb_info_window(GtkWidget *widget, gpointer data, guint action) {
+static void cb_info_window(GtkWidget *widget _U_, gpointer data, guint action _U_) {
   struct session_t *session = (struct session_t *) data;
   struct info_row_t rows[] = {
     { "Sound input device:", strdup(session->audio_device_name_in)},
     { "Input speed:", ltostr(session->audio_speed_in) },
     { "Input sample size (bits):", ltostr(session->audio_sample_size_in * 8)},
+    { "Input fragment size (samples):",
+      ltostr(session->fragment_size_in / session->audio_sample_size_in)},
     { "Input channels:", ltostr(1) },
+    { "", strdup("") },
+
     { "Sound output device:", strdup(session->audio_device_name_out)},
     { "Output speed:", ltostr(session->audio_speed_out) },
     { "Output sample size (bits):", ltostr(session->audio_sample_size_out*8)},
+    { "Input fragment size (samples):",
+      ltostr(session->fragment_size_out / session->audio_sample_size_out)},
     { "Output channels:", ltostr(1) },
-    { "ISDN device:", strdup(session->isdn_device_name) }
+    { "", strdup("") },
+
+    { "ISDN device:", strdup(session->isdn_device_name) },
+    { "ISDN speed (samples):", ltostr(8000) },
+    { "ISDN sample size (bite):", ltostr(8) },
+    { "ISDN fragment size (bytes):", ltostr(255) }
   };
   unsigned int i;
 
@@ -286,6 +237,7 @@ static void cb_info_window(GtkWidget *widget, gpointer data, guint action) {
   GtkWidget *label;
   GtkWidget *button_box;
   GtkWidget *button;
+  GtkWidget *separator;
 
   window = gtk_dialog_new();
   gtk_window_set_title(GTK_WINDOW(window), "Info");
@@ -298,15 +250,22 @@ static void cb_info_window(GtkWidget *widget, gpointer data, guint action) {
   gtk_widget_show(table);
 
   for (i = 0; i < sizeof(rows) / sizeof(struct info_row_t); i++) {
-    label = gtk_label_new(rows[i].tag);
-    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, i, i+1);
-    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-    gtk_widget_show(label);
+    if (strcmp(rows[i].tag, "")) { /* normal data */
+      label = gtk_label_new(rows[i].tag);
+      gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, i, i + 1);
+      gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+      gtk_widget_show(label);
 
-    label = gtk_label_new(rows[i].value);
-    gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, i, i+1);
-    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-    gtk_widget_show(label);
+      label = gtk_label_new(rows[i].value);
+      gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, i, i + 1);
+      gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+      gtk_widget_show(label);
+    } else { /* separator */
+      separator = gtk_hseparator_new();
+      gtk_table_attach(GTK_TABLE(table), separator, 0, 2, i, i + 1,
+		       GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 5);
+      gtk_widget_show(separator);
+    }
     
     free(rows[i].value); /* free stdrup() mem */
   }
@@ -333,136 +292,12 @@ static void cb_info_window(GtkWidget *widget, gpointer data, guint action) {
 /*
  * Options menu entries
  */
-
-/* Settings menu entry callback */
-static void cb_settings(GtkWidget *widget, gpointer data, guint action) {
-  struct session_t *session = (struct session_t *) data;
-  GtkWidget *window; /* the dialog window itself */
-  GtkWidget *button_box; /* box for buttons in standard way */
-  GtkWidget *button; /* action area buttons */
-  GtkWidget *table;  /* tables in frames */
-  GtkWidget *label;  /* different labels */
-  GtkWidget *frame;  /* the frames to separate msn/msns from sound devices */
-
-  GtkWidget *msn_entry;
-  GtkWidget *msns_entry;
-  GtkWidget *audio_device_name_in_entry;
-  GtkWidget *audio_device_name_out_entry;
-
-  window = gtk_dialog_new();
-  gtk_window_set_title(GTK_WINDOW(window), "ANT Settings");
-
-  /* vbox area */
-  frame = gtk_frame_new("ISDN");
-  gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), frame,
-		     TRUE, TRUE, 0);
-  gtk_widget_show(frame);
-
-  table = gtk_table_new(2,2, FALSE); /* 2 rows, 2 columns, not homogeneous */
-  gtk_container_add(GTK_CONTAINER(frame), table);
-  gtk_container_set_border_width(GTK_CONTAINER(table), 5);
-  gtk_table_set_row_spacings(GTK_TABLE(table), 5);
-  gtk_table_set_col_spacings(GTK_TABLE(table), 5);
-  gtk_widget_show(table);
-
-  label = gtk_label_new("Identifying MSN:");
-  gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,0,1);
-  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-  gtk_widget_show(label);
-
-  msn_entry = gtk_entry_new();
-  gtk_table_attach_defaults(GTK_TABLE(table), msn_entry, 1,2,0,1);
-  gtk_entry_set_text(GTK_ENTRY(msn_entry), session->msn);
-  gtk_widget_grab_focus(GTK_WIDGET(msn_entry));
-  gtk_widget_show(msn_entry);
-
-  label = gtk_label_new("Listen to MSNs:");
-  gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,1,2);
-  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-  gtk_widget_show(label);
-
-  msns_entry = gtk_entry_new();
-  gtk_table_attach_defaults(GTK_TABLE(table), msns_entry, 1,2,1,2);
-  gtk_entry_set_text(GTK_ENTRY(msns_entry), session->msns);
-  gtk_widget_show(msns_entry);
-
-  frame = gtk_frame_new("OSS");
-  gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), frame,
-		     TRUE, TRUE, 0);
-  gtk_widget_show(frame);
-
-  table = gtk_table_new(2,2, FALSE); /* 2 rows, 2 columns, not homogeneous */
-  gtk_container_add(GTK_CONTAINER(frame), table);
-  gtk_container_set_border_width(GTK_CONTAINER(table), 5);
-  gtk_table_set_row_spacings(GTK_TABLE(table), 5);
-  gtk_table_set_col_spacings(GTK_TABLE(table), 5);
-  gtk_widget_show(table);
-
-  label = gtk_label_new("Input sound device:");
-  gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,0,1);
-  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-  gtk_widget_show(label);
-
-  audio_device_name_in_entry = gtk_entry_new();
-  gtk_table_attach_defaults(GTK_TABLE(table), audio_device_name_in_entry,
-			    1,2,0,1);
-  gtk_entry_set_text(GTK_ENTRY(audio_device_name_in_entry),
-		     session->audio_device_name_in);
-  gtk_widget_show(audio_device_name_in_entry);
-
-  label = gtk_label_new("Output sound device:");
-  gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,1,2);
-  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-  gtk_widget_show(label);
-
-  audio_device_name_out_entry = gtk_entry_new();
-  gtk_table_attach_defaults(GTK_TABLE(table), audio_device_name_out_entry,
-			    1,2,1,2);
-  gtk_entry_set_text(GTK_ENTRY(audio_device_name_out_entry),
-		     session->audio_device_name_out);
-  gtk_widget_show(audio_device_name_out_entry);
-
-  /* action area */
-  button_box = gtk_hbutton_box_new();
-  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(window)->action_area),
-  		    button_box);
-  gtk_widget_show(button_box);
-
-  button = gtk_button_new_with_label("OK");
-  gtk_box_pack_start_defaults(GTK_BOX(button_box), button);
-  gtk_object_set_data(GTK_OBJECT(button), "msn_entry", (gpointer) msn_entry);
-  gtk_object_set_data(GTK_OBJECT(button), "msns_entry", (gpointer) msns_entry);
-  gtk_object_set_data(GTK_OBJECT(button), "audio_device_name_in_entry",
-		      (gpointer) audio_device_name_in_entry);
-  gtk_object_set_data(GTK_OBJECT(button), "audio_device_name_out_entry",
-		      (gpointer) audio_device_name_out_entry);
-  gtk_object_set_data(GTK_OBJECT(button), "dialog_window",
-		      (gpointer) window);
-  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		     GTK_SIGNAL_FUNC(cb_settings_ok), session);
-  gtk_widget_show(button);
-
-  button = gtk_button_new_with_label("Cancel");
-  gtk_box_pack_start_defaults(GTK_BOX(button_box), button);
-  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
-			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
-			    GTK_OBJECT(window));
-  gtk_widget_show(button);
-  
-  gtk_window_set_modal(GTK_WINDOW(window), TRUE);
-
-  /* show everything */
-  gtk_widget_show(window);
-}
-
 /*
  * Help menu entries
  */
 
 /* the about menu entry callback */
-static void cb_about(GtkWidget *widget, gpointer data, guint action) {
+static void cb_about(GtkWidget *widget _U_, gpointer data _U_, guint action _U_) {
   GtkWidget *window = ok_dialog_get("About ANT",
 			"ANT (ANT is Not a Telephone) Version " VERSION "\n"
 			"Copyright 2002 Roland Stigge\n\n"
@@ -473,13 +308,14 @@ static void cb_about(GtkWidget *widget, gpointer data, guint action) {
 			"audio capable I4L ISDN device\n\n"
 			"Contact:\n"
 			"Roland Stigge, stigge@epost.de\n"
-                        "http://www.antcom.de/", GTK_JUSTIFY_CENTER);
+			"http://www.antcom.de/"
+				    , GTK_JUSTIFY_CENTER);
 
   gtk_widget_show(window);
 }
 
 /* the about menu entry callback */
-static void cb_license(GtkWidget *widget, gpointer data, guint action) {
+static void cb_license(GtkWidget *widget _U_, gpointer data _U_, guint action _U_) {
   GtkWidget *window = ok_dialog_get("ANT License",
     "ANT (ANT is Not a Telephone)\n"
     "Copyright (C) 2002 Roland Stigge\n"
@@ -529,7 +365,7 @@ GtkWidget *get_main_menu(GtkWidget *window, struct session_t *session) {
                             NULL,     llcheck_toggle_cb,0,     "<CheckItem>" },
 
     { "/_Options",          NULL,     NULL,          0,        "<Branch>"    },
-    { "/Options/_Settings", NULL,     cb_settings,   0,        NULL          },
+    { "/Options/_Settings", NULL,     gtksettings_cb,0,        NULL          },
 
     { "/_Help",             NULL,     NULL,          0,        "<LastBranch>"},
     { "/Help/_About",       NULL,     cb_about,      0,        NULL          },
@@ -559,9 +395,11 @@ GtkWidget *get_main_menu(GtkWidget *window, struct session_t *session) {
   session->menuitem_line_check = gtk_item_factory_get_item(item_factory,
 				   "/Phone/Line Level Check");
   gtk_check_menu_item_set_active(
-    GTK_CHECK_MENU_ITEM(session->cid_check_menu_item), TRUE);
+    GTK_CHECK_MENU_ITEM(session->cid_check_menu_item),
+    session->option_show_callerid);
   gtk_check_menu_item_set_active(
-    GTK_CHECK_MENU_ITEM(session->llcheck_check_menu_item), TRUE);
+    GTK_CHECK_MENU_ITEM(session->llcheck_check_menu_item),
+    session->option_show_llcheck);
   
   return gtk_item_factory_get_widget(item_factory, "<main>");
 }
@@ -686,7 +524,7 @@ GtkWidget *get_dial_box(struct session_t *session) {
 }
 
 /*
- * main function for gtk gui
+ * main function for gtk GUI
  *
  * returns int to be returned from main()
  */
@@ -698,6 +536,9 @@ int main_gtk(struct session_t *session) {
   GtkWidget *dialbox;
   GtkWidget *cidbox;
 
+  GtkRcStyle *rc_style;
+  GdkColor color;
+
   /* the main window */
   session->main_window = main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(main_window), "ANT");
@@ -755,6 +596,24 @@ int main_gtk(struct session_t *session) {
 		   session->llcheck_out, FALSE, FALSE, 0);
   gtk_widget_show(session->llcheck_out);
 
+  /* audio warning inside status bar */
+  session->audio_warning = gtk_statusbar_new();
+  gdk_color_parse("#B00000", &color);
+  rc_style = gtk_rc_style_new();
+  rc_style->fg[GTK_STATE_NORMAL] = color;
+  rc_style->color_flags[GTK_STATE_NORMAL] |= GTK_RC_FG;
+  gtk_widget_modify_style(GTK_STATUSBAR(session->audio_warning)->label,
+			  rc_style);
+  gtk_rc_style_unref(rc_style);
+
+  gtk_box_pack_end(GTK_BOX(session->status_bar),
+		   session->audio_warning, FALSE, FALSE, 0);
+  session->audio_context_id =
+    gtk_statusbar_get_context_id(GTK_STATUSBAR(session->audio_warning),
+				 "audio");
+  gtk_statusbar_push(GTK_STATUSBAR(session->audio_warning),
+		     session->audio_context_id, "");
+  
   /* main menu bar */
   main_menu = get_main_menu(main_window, session); /* main menu */
   gtk_widget_show(main_menu);
@@ -774,12 +633,14 @@ int main_gtk(struct session_t *session) {
   /* show everything */
   gtk_widget_show(main_window);
 
-  /* set up for initial state */
-  session_set_state(session, session->state); /* state is already in session */
+  settings_history_read(session); /* get history from home dir dotfile */
 
   /* connect isdn and sound input handlers to gdk pseudo select */
   session_io_handlers_start(session);
 
+  /* set up for initial state */
+  session_set_state(session, STATE_READY); /* state is already in session */
+
   /* set up additional handlers */
   gtk_timeout_add(TIMER_DELAY, timeout_callback, (gpointer) session);
   signal(SIGINT, &terminate_signal_callback);
