@@ -7,11 +7,21 @@
  *
  */
 
+#include "config.h"
+
 #include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <termios.h>
-#include <fcntl.h>
+#ifdef HAVE_STDLIB_H
+  #include <stdlib.h>
+#endif
+#ifdef HAVE_UNISTD_H
+  #include <unistd.h>
+#endif
+#ifdef HAVE_TERMIOS_H
+  #include <termios.h>
+#endif
+#ifdef HAVE_FCNTL_H
+  #include <fcntl.h>
+#endif
 #include <string.h>
 #include <signal.h>
 #include <errno.h>
