@@ -7,11 +7,17 @@
  *
  */
 
+#include "config.h"
+
 /* regular GNU system includes */
 #include <string.h>
 #include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
+#ifdef HAVE_STDLIB_H
+  #include <stdlib.h>
+#endif
+#ifdef HAVE_UNISTD_H
+  #include <unistd.h>
+#endif
 #include <math.h>
 #include <time.h>
 
@@ -24,6 +30,7 @@
 #include "util.h"
 #include "callerid.h"
 #include "llcheck.h"
+#include "settings.h"
 
 /*
  * This is our session. Currently just one globally.
@@ -31,56 +38,24 @@
 struct session_t session;
 
 struct state_data_t state_data[] = {
-  { "Ready",          "Dial",    1, "Hang up", 0 }, /* STATE_READY        */
-  { "RING",           "Answer",  1, "Reject",  1 }, /* STATE_RINGING      */
-  { "Dialing",        "Pick up", 0, "Cancel",  1 }, /* STATE_DIALING      */
-  { "B-Channel open", "Pick up", 0, "Hang up", 1 }  /* STATE_CONVERSATION */
+  { "Ready",          "Dial",    1, "Hang up", 0 }, /* STATE_READY         */
+  { "RING",           "Answer",  1, "Reject",  1 }, /* STATE_RINGING       */
+  { "RING",           "Answer",  1, "Reject",  1 }, /* STATE_RINGING_QUIET */
+  { "Dialing",        "Pick up", 0, "Cancel",  1 }, /* STATE_DIALING       */
+  { "B-Channel open", "Pick up", 0, "Hang up", 1 }, /* STATE_CONVERSATION  */
+  { "Setup",          "Pick up", 0, "Hang up", 0 }  /* STATE_SERVICE       */
 };
 
 /*
- * initialize a session (isdn device, audio device)
+ * Simply opens audio devices for specified session
  *
- * input:
- *   session: empty struct waiting for work
- *   audio_device_name_in, audio_device_name_out: name(s) of audio device(s)
- *   msn: msn to use
- *
- * output: session: initialized session struct
- *
- * returns 0 un success, -1 otherwise
+ * returns 0 on success, -1 on error
  */
-int session_init(struct session_t *session,
-		 char *audio_device_name_in,
-		 char *audio_device_name_out,
-		 char *msn, char *msns) {
-
-  session->audio_speed_in = ISDN_SPEED;  /* default audio speed */
-  session->audio_speed_out = ISDN_SPEED;
-
-  /* audio device buffer fragment sizes */
-  session->fragment_size_in = DEFAULT_FRAGMENT_SIZE;
-  session->fragment_size_out = DEFAULT_FRAGMENT_SIZE;
-
-  session->format_priorities = default_audio_priorities;
-
-  session->msn = strdup(msn);
-  session->msns = strdup(msns);
-
-  /* open audio device(s) */
-  if (debug) {
-    if (strcmp(audio_device_name_in, audio_device_name_out))
-      {
-	/* different devices */
-	fprintf(stderr, "Initializing %s and %s...\n",
-		audio_device_name_in, audio_device_name_out);
-      } else {
-	fprintf(stderr, "Initializing %s ...\n",
-		audio_device_name_in);
-      }
-  }
-  
-  if (open_audio_devices(audio_device_name_in,
-			 audio_device_name_out,
+int session_audio_open(struct session_t *session) {
+  if (debug)
+    fprintf(stderr, "session_audio_open: Opening audio device(s).\n");
+  if (open_audio_devices(session->audio_device_name_in,
+			 session->audio_device_name_out,
 			 1,
 			 session->format_priorities,
 			 &session->audio_fd_in,
@@ -91,13 +66,64 @@ int session_init(struct session_t *session,
 			 &session->audio_format_out,
 			 &session->audio_speed_in,
 			 &session->audio_speed_out)) {
+    return -1;
+  }
+  return 0;
+}
+
+/*
+ * Closes audio devices for specified session
+ *
+ * returns 0 on success, -1 on error
+ */
+int session_audio_close(struct session_t *session) {
+  if (debug)
+    printf("session_audio_close: Closing audio device(s).\n");
+  if (close_audio_devices(session->audio_fd_in, session->audio_fd_out)) {
+    return -1;
+  }
+  return 0;
+}
+
+/*
+ * init audio devices in session
+ *
+ * input: session->audio_device_name_in, session->audio_device_name_out
+ *
+ * returns 0 on success, -1 otherwise
+ *
+ * output: on success, session->audio_LUT* and session->audio_*buf are
+ *         allocated (and set, if appropriate)
+ */
+int session_audio_init(struct session_t *session) {
+  /* message: open audio device(s) */
+  if (debug) {
+    if (strcmp(session->audio_device_name_in, session->audio_device_name_out))
+      {
+	/* different devices */
+	fprintf(stderr, "Initializing %s and %s...\n",
+		session->audio_device_name_in, session->audio_device_name_out);
+      } else {
+	fprintf(stderr, "Initializing %s ...\n",
+		session->audio_device_name_in);
+      }
+  }
+  
+  /* other options */
+  session->audio_speed_in = ISDN_SPEED;  /* default audio speed */
+  session->audio_speed_out = ISDN_SPEED;
+
+  /* audio device buffer fragment sizes */
+  session->fragment_size_in = DEFAULT_FRAGMENT_SIZE;
+  session->fragment_size_out = DEFAULT_FRAGMENT_SIZE;
+
+  session->format_priorities = default_audio_priorities;
+
+  if (session_audio_open(session)) {
     fprintf(stderr, "Error initializing audio device(s).\n");
     return -1;
   }
 
-  session->audio_device_name_in = strdup(audio_device_name_in);
-  session->audio_device_name_out = strdup(audio_device_name_out);
-
   session->ratio_in = (double)session->audio_speed_out / ISDN_SPEED;
   session->ratio_out = (double)ISDN_SPEED / session->audio_speed_in;
 
@@ -114,6 +140,21 @@ int session_init(struct session_t *session,
   session->audio_sample_size_out =
     sample_size_from_format(session->audio_format_out);
 
+  /* allocate buffers */
+  session->audio_inbuf  = (unsigned char *)malloc(session->fragment_size_in);
+  session->audio_outbuf = (unsigned char *)malloc(session->fragment_size_out);
+
+  return 0;
+}
+
+/*
+ * init isdn in session
+ *
+ * input: session->msn, session->msns
+ *
+ * returns 0 on success, -1 otherwise
+ */
+int session_isdn_init(struct session_t *session) {
   /* open and init isdn device */
   if (debug)
     fprintf(stderr, "Initializing ISDN device...\n");
@@ -131,8 +172,6 @@ int session_init(struct session_t *session,
   session->isdn_inbuf_size = session->isdn_outbuf_size = DEFAULT_ISDNBUF_SIZE;
 
   /* allocate buffers */
-  session->audio_inbuf  = (unsigned char *)malloc(session->fragment_size_in);
-  session->audio_outbuf = (unsigned char *)malloc(session->fragment_size_out);
   session->isdn_inbuf   = (unsigned char *)malloc(session->isdn_inbuf_size);
   session->isdn_outbuf  = (unsigned char *)malloc(session->isdn_outbuf_size);
   
@@ -142,47 +181,49 @@ int session_init(struct session_t *session,
     return -1;
   }
 
-  session->state = STATE_READY; /* initial state */
-
   session->isdn_inbuf_len = 0;
   session->isdn_inbuf[0] = 1;
 
-  session->dial_number_history = NULL;
-  session->dial_number_history = g_list_append(session->dial_number_history,
-  					       strdup(""));
-  /* XXX: read history, later save it */
-  session->dial_number_history_maxlen = 10; /* XXX: make configurable */
-
-  session->effect = EFFECT_NONE;
-
   return 0;
 }
 
 /*
- * de-initialize a session (isdn device, audio device)
+ * close audio device(s) and clean up (deallocate buffers)
  *
- * input: session: session to de-initialize
- *
- * returns 0 un success, -1 otherwise
+ * returns 0 on success, -1 otherwise
  */
-/* just a helper function */
-void free_g_list_element(gpointer data, gpointer user_data) {
-  free(data);
-}
-int session_deinit(struct session_t *session) {
-  
-  /* free dial_number_history */
-  g_list_foreach(session->dial_number_history, free_g_list_element, NULL);
-  g_list_free(session->dial_number_history);
-
-  /* assume: STATE_READY */
-
+int session_audio_deinit(struct session_t *session) {
   /* free allocated buffers */
+  
   free(session->audio_inbuf);
   free(session->audio_outbuf);
+  
+  free(session->audio_LUT_in);
+  free(session->audio_LUT_out);
+  free(session->audio_LUT_generate);
+  free(session->audio_LUT_analyze);
+  
+  /* close audio device(s) */
+  if (debug)
+    fprintf(stderr, "Closing sound device(s)...\n");
+  if (session_audio_close(session)) {
+    fprintf(stderr, "Error closing sound device(s).\n");
+    return -1;
+  }
+
+  return 0;
+}
+
+/*
+ * close isdn device and clean up (deallocate buffers)
+ *
+ * returns 0 on success, -1 otherwise
+ */
+int session_isdn_deinit(struct session_t *session) {
+  /* free allocated buffers */
   free(session->isdn_inbuf);
   free(session->isdn_outbuf);
-
+  
   if (debug)
     fprintf(stderr, "Closing ISDN device...\n");
   /* de-init / restore isdn device */
@@ -196,23 +237,113 @@ int session_deinit(struct session_t *session) {
     fprintf(stderr, "Error closing isdn device.\n");
   }
   
-  free(session->audio_LUT_in);
-  free(session->audio_LUT_out);
-  free(session->audio_LUT_generate);
-  free(session->audio_LUT_analyze);
+  return 0;
+}
 
-  /* close audio device(s) */
-  if (debug)
-    fprintf(stderr, "Closing sound device(s)...\n");
-  if (close_audio_devices(session->audio_fd_in, session->audio_fd_out)) {
-    fprintf(stderr, "Error closing sound device(s).\n");
-    return -1;
+/*
+ * initialize a session (isdn device, audio device) and read options file
+ *
+ * input:
+ *   session: empty struct waiting for work
+ *   audio_device_name_in, audio_device_name_out: name(s) of audio device(s)
+ *   msn: msn to use
+ *
+ * NOTE: The latter 4 parameters are only the requested ones. When they are
+ *       overridden by the options file, the resulting values will be different
+ *
+ * output: session: initialized session struct
+ *
+ * returns 0 on success, -1 otherwise
+ */
+int session_init(struct session_t *session,
+		 char *audio_device_name_in,
+		 char *audio_device_name_out,
+		 char *msn, char *msns) {
+
+  /*
+   * first: set all defaults possibly overridden by options file
+   */
+
+  session->dial_number_history = NULL;
+  session->dial_number_history = g_list_append(session->dial_number_history,
+  					       strdup(""));
+  session->dial_number_history_maxlen = 10; /* config overrides this */
+
+  /* default: save options automatically (on exit) */
+  session->option_save_options = 1;
+  session->option_release_devices = 0;
+  session->option_show_llcheck = 1;
+  session->option_show_callerid = 1;
+
+  session->audio_device_name_in = strdup(audio_device_name_in);
+  session->audio_device_name_out = strdup(audio_device_name_out);
+  session->msn = strdup(msn);
+  session->msns = strdup(msns);
+
+  settings_options_read(session); /* override defaults analyzing options file*/
+
+  /* command line configurable parameters: set to hard coded defaults
+     if no setting was made (either at command line or in options file) */
+  if (!strcmp(session->audio_device_name_in, "")) {
+    free(session->audio_device_name_in);
+    session->audio_device_name_in = strdup(DEFAULT_AUDIO_DEVICE_NAME_IN);
+  }
+  if (!strcmp(session->audio_device_name_out, "")) {
+    free(session->audio_device_name_out);
+    session->audio_device_name_out = strdup(DEFAULT_AUDIO_DEVICE_NAME_OUT);
+  }
+  if (!strcmp(session->msn, "")) {
+    free(session->msn);
+    session->msn = strdup(DEFAULT_MSN);
+  }
+  if (!strcmp(session->msns, "")) {
+    free(session->msns);
+    session->msns = strdup(DEFAULT_MSNS);
   }
 
-  free(session->msn);
-  free(session->msns);
+  /* setup audio and isdn */
+  if (session_audio_init(session) < 0) return -1;
+  if (session_isdn_init(session) < 0) return -1;
+
+  session->state = STATE_SERVICE; /* initial state */
+
+  session->effect = EFFECT_NONE;
+  
+  return 0;
+}
+
+/*
+ * helper function to free single element data from g_list 
+ * (used by session_deinit)
+ */
+void free_g_list_element(gpointer data, gpointer user_data _U_) {
+  free(data);
+}
+
+/*
+ * de-initialize a session (isdn device, audio device)
+ *
+ * input: session: session to de-initialize
+ *
+ * returns 0 un success, -1 otherwise
+ */
+int session_deinit(struct session_t *session) {
+  
+  if (session->option_save_options) settings_options_write(session);
+
+  /* free dial_number_history */
+  g_list_foreach(session->dial_number_history, free_g_list_element, NULL);
+  g_list_free(session->dial_number_history);
+
+  /* close devices and clean up (buffers) */
+  if (session_isdn_deinit(session) < 0) return -1;
+  if (session_audio_deinit(session) < 0) return -1;
+
+  /* clean up pre-set options */
   free(session->audio_device_name_in);
   free(session->audio_device_name_out);
+  free(session->msn);
+  free(session->msns);
 
   return 0;
 }
@@ -221,10 +352,14 @@ int session_deinit(struct session_t *session) {
  * set up handlers for audio/ISDN input
  */
 void session_io_handlers_start(struct session_t *session) {
-  session->gdk_audio_input_tag = gdk_input_add(session->audio_fd_in,
-					       GDK_INPUT_READ,
-					       gdk_handle_audio_input,
-					       (gpointer) session);
+  if (!(session->option_release_devices &&
+	(session->state == STATE_READY ||
+	 session->state == STATE_RINGING_QUIET))) {
+    session->gdk_audio_input_tag = gdk_input_add(session->audio_fd_in,
+						 GDK_INPUT_READ,
+						 gdk_handle_audio_input,
+						 (gpointer) session);
+  }
   session->gdk_isdn_input_tag = gdk_input_add(session->isdn_fd,
 					      GDK_INPUT_READ,
 					      gdk_handle_isdn_input,
@@ -235,7 +370,11 @@ void session_io_handlers_start(struct session_t *session) {
  * remove handlers for audio/ISDN input
  */
 void session_io_handlers_stop(struct session_t *session) {
-  gdk_input_remove(session->gdk_audio_input_tag);
+  if (!(session->option_release_devices &&
+	(session->state == STATE_READY ||
+	 session->state == STATE_RINGING_QUIET))) {
+    gdk_input_remove(session->gdk_audio_input_tag);
+  }
   gdk_input_remove(session->gdk_isdn_input_tag);
 }
 
@@ -244,26 +383,28 @@ void session_io_handlers_stop(struct session_t *session) {
  *
  * assumes audio in open, initialized (possibly used) state
  *
- * WARNING: stop I/O handlers first!
+ * WARNING: * Stop I/O handlers first!
+ *          * Only resets currently used device(s). To use another device,
+ *            use session_audio_deinit() and session_audio_init()
+ *
+ * returns 0 on success, -1 on error
  */
-void session_reset_audio(struct session_t *session) {
-  if (close_audio_devices(session->audio_fd_in, session->audio_fd_out)) {
-    fprintf(stderr, "Error closing audio device(s) while resetting.\n");
-  }
-  if (open_audio_devices(session->audio_device_name_in,
-			 session->audio_device_name_out,
-			 1,
-			 session->format_priorities,
-			 &session->audio_fd_in,
-			 &session->audio_fd_out,
-			 &session->fragment_size_in,
-			 &session->fragment_size_out,
-			 &session->audio_format_in,
-			 &session->audio_format_out,
-			 &session->audio_speed_in,
-			 &session->audio_speed_out)) {
-    fprintf(stderr, "Error reopening audio device(s) while resetting.\n");
+int session_reset_audio(struct session_t *session) {
+  int result = 0;
+  
+  if (!(session->option_release_devices &&
+	(session->state == STATE_READY ||
+	 session->state == STATE_RINGING_QUIET))) {
+    if (session_audio_close(session)) {
+      fprintf(stderr, "Error closing audio device(s) while resetting.\n");
+      result = -1;
+    }
+    if (session_audio_open(session)) {
+      fprintf(stderr, "Error reopening audio device(s) while resetting.\n");
+      result = -1;
+    }
   }
+  return result;
 }
 
 /*
@@ -402,6 +543,10 @@ void session_start_conversation(struct session_t *session) {
     session->vcon_time = time(NULL); /* for caller id monitor */
     session_init_conversation(session); /* init some conversation variables */
     
+    session_io_handlers_stop(session);
+    session_reset_audio(session);
+    session_io_handlers_start(session);
+
     /* start with first block to start recording */
     process_audio_source(session);
   }
@@ -410,8 +555,8 @@ void session_start_conversation(struct session_t *session) {
 /*
  * To be called when we can write another block to sound device
  */
-void session_effect_callback(gpointer data, gint fd,
-			     GdkInputCondition condition) {
+void session_effect_callback(gpointer data, gint fd _U_,
+			     GdkInputCondition condition _U_) {
 #define RING_PERIOD 4
 #define RING_LENGTH 1.3
 #define RING_FREQUENCY 1300
@@ -541,9 +686,47 @@ void session_effect_stop(struct session_t *session) {
 }
 
 /*
- * Sets new state in session and GUI
+ * Sets status bar audio state (e.g. "AUDIO OFF")
+ * hide if note is ""
  */
-void session_set_state(struct session_t *session, enum state_t state) {
+void session_audio_notify(struct session_t *session, char *note) {
+  gtk_widget_hide(session->audio_warning);
+  if (*note) {
+    gtk_statusbar_pop(GTK_STATUSBAR(session->audio_warning),
+		      session->audio_context_id);
+    gtk_statusbar_push(GTK_STATUSBAR(session->audio_warning),
+		       session->audio_context_id, note);
+    gtk_widget_show(session->audio_warning);
+  }
+}
+
+/*
+ * Sets new state in session and GUI (also handles audio state)
+ *
+ * returns 0 on success, -1 otherwise (can't open audio device)
+ */
+int session_set_state(struct session_t *session, enum state_t state) {
+  int result = 0;
+
+  /* open / close audio when needed, set state */
+  session_io_handlers_stop(session);
+  if (session->option_release_devices && state != session->state) {
+    if (state == STATE_READY && session->state != STATE_RINGING_QUIET) {
+      /* release */
+      session_audio_close(session);
+    } else if ((session->state == STATE_READY ||
+		session->state == STATE_RINGING_QUIET) &&
+	       state != STATE_READY && state != STATE_RINGING_QUIET) {
+      /* (try to) resume */
+      if (session_audio_open(session)) {
+	state = session->state;
+	result = -1;
+      }
+    }
+  }
+  session->state = state;
+  session_io_handlers_start(session);
+
   /* some menu items are selected only in STATE_READY */
   gtk_widget_set_sensitive(session->menuitem_settings, state == STATE_READY);
   gtk_widget_set_sensitive(session->menuitem_line_check, state == STATE_READY);
@@ -558,20 +741,34 @@ void session_set_state(struct session_t *session, enum state_t state) {
     if (session->effect == EFFECT_NONE)
       session_effect_start(session, EFFECT_RING);
     break;
+  case STATE_RINGING_QUIET:
+    break;
   case STATE_READY:
     gtk_widget_grab_focus(GTK_WIDGET(GTK_COMBO(session->dial_number_box)
 				     ->entry));
-    /* no break */
+    if (session->effect != EFFECT_NONE)
+      session_effect_stop(session);
+    break;
   case STATE_CONVERSATION:
     if (session->effect != EFFECT_NONE)
       session_effect_stop(session);
     break;
+  case STATE_SERVICE:
+    break;
   default:
-    ;
+    fprintf(stderr, "Warning: session_set_state: Unhandled state.\n");
+  }
+
+  /* audio on / off notify */
+  if (session->option_release_devices) {
+    session_audio_notify(session,
+			 state == STATE_READY || state == STATE_RINGING_QUIET ?
+			 "AUDIO OFF" : "AUDIO ON");
+  } else {
+    session_audio_notify(session, "");
   }
 
   /* status line */
-  session->state = state;
   gtk_statusbar_pop(GTK_STATUSBAR(session->status_bar),
 		    session->phone_context_id);
   gtk_statusbar_push(GTK_STATUSBAR(session->status_bar),
@@ -591,6 +788,8 @@ void session_set_state(struct session_t *session, enum state_t state) {
     llcheck_bar_reset(session->llcheck_in);
     llcheck_bar_reset(session->llcheck_out);
   }
+
+  return result;
 }
 
 /*
@@ -600,19 +799,21 @@ void session_set_state(struct session_t *session, enum state_t state) {
  *        fd:        file descriptor where we got the input from
  *        condition: will be GDK_INPUT_READ in this case
  */
-void gdk_handle_isdn_input(gpointer data, gint fd,
-			   GdkInputCondition condition) {
+void gdk_handle_isdn_input(gpointer data, gint fd _U_,
+			   GdkInputCondition condition _U_) {
   struct session_t *session = (struct session_t *) data;
 
   switch (session->state) {
   case STATE_READY: /* we are in command mode */
     if (isdn_try_read_line(session)){ /* got new line: something happened */
       if (!strncmp(session->isdn_inbuf, "RING/", 5)) { /* -> RINGING state */
-	session_set_state(session, STATE_RINGING);
+	if (session_set_state(session, STATE_RINGING))
+	  session_set_state(session, STATE_RINGING_QUIET);
+
 	session->isdn_inbuf[session->isdn_inbuf_len - 2] = 0;
-	
 	/* caller id update */
 	cid_add_line(session, CALL_IN, NULL, &session->isdn_inbuf[5]);
+
       } else { /* something else */
       }
     }
@@ -633,8 +834,10 @@ void gdk_handle_isdn_input(gpointer data, gint fd,
     }
     break;
   case STATE_RINGING:
+  case STATE_RINGING_QUIET:
     if (isdn_try_read_line(session)){ /* got new line: something happened */
       if (strstr(session->isdn_inbuf, "VCON\r\n")) { /* let's go! */
+	/* will only come in STATE_RINGING */
 	session_start_conversation(session); /* including state transition */
       } else if (strstr(session->isdn_inbuf, "CALLER NUMBER: ")) {
 	/* got Caller ID */
@@ -666,8 +869,13 @@ void gdk_handle_isdn_input(gpointer data, gint fd,
       cid_call_finished(session, NULL);
     }
     break;
+  case STATE_SERVICE:
+    if (debug)
+      fprintf(stderr, "Note: Got ISDN input in service mode.\n");
+    break;
   default:
-    fprintf(stderr, "Warning: Unknown session state.\n");
+    fprintf(stderr,
+	    "Warning: gdk_handle_isdn_input: Unknown session state.\n");
   }
 }
 
@@ -678,8 +886,8 @@ void gdk_handle_isdn_input(gpointer data, gint fd,
  *        fd:        file descriptor where we got the input from
  *        condition: will be GDK_INPUT_READ in this case
  */
-void gdk_handle_audio_input(gpointer data, gint fd,
-			    GdkInputCondition condition) {
+void gdk_handle_audio_input(gpointer data, gint fd _U_,
+			    GdkInputCondition condition _U_) {
   struct session_t *session = (struct session_t *) data;
 
   switch (session->state) {
@@ -704,6 +912,14 @@ void gdk_handle_audio_input(gpointer data, gint fd,
     read(session->audio_fd_in,
 	 session->audio_inbuf, session->fragment_size_in);
     break;
+  case STATE_RINGING_QUIET:
+    if (debug > 1)
+      fprintf(stderr,
+	      "Warning: Got audio input in QUIET ringing mode.\n");
+    /* flush audio input */
+    read(session->audio_fd_in,
+	 session->audio_inbuf, session->fragment_size_in);
+    break;
   case STATE_CONVERSATION:
     process_audio_source(session);
     if (session->samples_out >= session->audio_speed_in)
@@ -725,8 +941,19 @@ void gdk_handle_audio_input(gpointer data, gint fd,
       cid_call_finished(session, NULL);
     }
     break;
+  case STATE_SERVICE:
+    if (debug > 1)
+      fprintf(stderr, "Warning: Got audio input in service mode (ALSA?).\n");
+    /* flush audio input */
+    read(session->audio_fd_in,
+	 session->audio_inbuf, session->fragment_size_in);
+    break;
   default:
-    fprintf(stderr, "Warning: Unknown session state.\n");
+    fprintf(stderr,
+	    "Warning: gdk_handle_audio_input: Unknown session state.\n");
+    /* flush audio input */
+    read(session->audio_fd_in,
+	 session->audio_inbuf, session->fragment_size_in);
   }
 }
 
@@ -736,10 +963,9 @@ void gdk_handle_audio_input(gpointer data, gint fd,
  * input: widget: the button
  *        data:   will be a (struct session_t *)
  */
-void gtk_handle_pick_up_button(GtkWidget *widget, gpointer data) {
+void gtk_handle_pick_up_button(GtkWidget *widget _U_, gpointer data) {
   struct session_t *session = (struct session_t *) data;
   char *s; /* the modem dial string */
-  char *temp;
   char *number; /* the number to dial "inside" gtk (entry) */
   char *clear_number; /* number after un_vanity() */
   int result;
@@ -752,34 +978,26 @@ void gtk_handle_pick_up_button(GtkWidget *widget, gpointer data) {
     clear_number = un_vanity(strdup(number));
     if ((s = (char*) malloc(strlen(clear_number) + 5)) &&
 	strcmp(clear_number, "")) {
-      snprintf(s, strlen(clear_number) + 5, "ATD%s\n", clear_number);
+      if (!session_set_state(session, STATE_DIALING)) {
+	/* dial only if audio is on etc. */
+	snprintf(s, strlen(clear_number) + 5, "ATD%s\n", clear_number);
+	
+	tty_clear(session->isdn_fd);
+	result = tty_write(session->isdn_fd, s);
+	free(s);
+	if (result) {
+	  fprintf(stderr, "Error dialing.\n");
+	} else {
 
-      tty_clear(session->isdn_fd);
-      result = tty_write(session->isdn_fd, s);
-      free(s);
-      if (result) {
-	fprintf(stderr, "Error dialing.\n");
+	  /* update dial combo box */
+	  session_history_add(session, number);
+	  
+	  /* caller id update */
+	  cid_add_line(session, CALL_OUT, session->msn, clear_number);
+	}
+      } else {
+	show_audio_error_dialog();
       }
-
-      session_set_state(session, STATE_DIALING);
-
-      /* update dial combo box */
-      temp = strdup(number);
-      session->dial_number_history = g_list_insert(
-				       session->dial_number_history, temp, 0);
-      while (g_list_length(session->dial_number_history) >
-	     session->dial_number_history_maxlen) {
-	free(g_list_nth_data(session->dial_number_history,
-			     g_list_length(session->dial_number_history) - 1));
-	session->dial_number_history = g_list_remove_link(
-	    session->dial_number_history,
-	    g_list_last(session->dial_number_history));
-      }
-      gtk_combo_set_popdown_strings(GTK_COMBO(session->dial_number_box),
-				    session->dial_number_history);
-
-      /* caller id update */
-      cid_add_line(session, CALL_OUT, session->msn, clear_number);
     }
 
     free(clear_number);
@@ -792,10 +1010,22 @@ void gtk_handle_pick_up_button(GtkWidget *widget, gpointer data) {
     if (tty_write(session->isdn_fd, "ATA\n"))
       fprintf(stderr, "Error answering call.\n");
     break;
+  case STATE_RINGING_QUIET:
+    if (!session_set_state(session, STATE_RINGING)) {
+      tty_clear(session->isdn_fd);
+      if (tty_write(session->isdn_fd, "ATA\n"))
+	fprintf(stderr, "Error answering call.\n");
+    } else {
+      show_audio_error_dialog();
+    }
+    break;
   case STATE_CONVERSATION: /* channel already working */
     break;
+  case STATE_SERVICE:
+    break;
   default:
-    fprintf(stderr, "Warning: Unknown session state.\n");
+    fprintf(stderr,
+	    "Warning: gtk_handle_pick_up_button: Unknown session state.\n");
   }
 }
 
@@ -805,7 +1035,7 @@ void gtk_handle_pick_up_button(GtkWidget *widget, gpointer data) {
  * input: widget: the button, NULL when called directly (on exit)
  *        data:   will be a (struct session_t *)
  */
-void gtk_handle_hang_up_button(GtkWidget *widget, gpointer data) {
+void gtk_handle_hang_up_button(GtkWidget *widget _U_, gpointer data) {
   struct session_t *session = (struct session_t *) data;
 
   switch (session->state) {
@@ -819,6 +1049,7 @@ void gtk_handle_hang_up_button(GtkWidget *widget, gpointer data) {
     cid_call_finished(session, "(ABORTED)");
     break;
   case STATE_RINGING: /* reject call */
+  case STATE_RINGING_QUIET: /* reject call */
     tty_clear(session->isdn_fd);
     if (tty_write(session->isdn_fd, "ATH\n"))
       fprintf(stderr, "Error answering call.\n");
@@ -831,7 +1062,65 @@ void gtk_handle_hang_up_button(GtkWidget *widget, gpointer data) {
     session_set_state(session, STATE_READY);
     cid_call_finished(session, NULL);
     break;
+  case STATE_SERVICE:
+    break;
   default:
-    fprintf(stderr, "Warning: Unknown session state.\n");
+    fprintf(stderr,
+	    "Warning: gtk_handle_hang_up_button: Unknown session state.\n");
   }
 }
+
+/*
+ * cut session->dial_number_history to specified size
+ * (session->dial_number_history_maxlen)
+ * -> and redisplay in session->dial_number_box
+ */
+static void session_history_normalize(struct session_t *session) {
+  /* remove leading blank line(s) if possible */
+  /*
+  while (g_list_length(session->dial_number_history) > 1 &&
+	 !strcmp(g_list_nth_data(session->dial_number_history, 0), "")) {
+    free(g_list_nth_data(session->dial_number_history, 0));
+    session->dial_number_history = g_list_remove_link(
+	    session->dial_number_history,
+	    g_list_first(session->dial_number_history));
+  }
+  */
+
+  /* cut size if needed */
+  while (g_list_length(session->dial_number_history) >
+	 session->dial_number_history_maxlen) {
+    free(g_list_nth_data(session->dial_number_history,
+			 g_list_length(session->dial_number_history) - 1));
+    session->dial_number_history = g_list_remove_link(
+	    session->dial_number_history,
+	    g_list_last(session->dial_number_history));
+  }
+  gtk_combo_set_popdown_strings(GTK_COMBO(session->dial_number_box),
+				session->dial_number_history);
+}
+
+/*
+ * Add line to history of dial number combo box at start (first row)
+ * and care about maximum size of history
+ *
+ * number will be copied, so caller has to care about it's associated memory
+ */
+void session_history_add(struct session_t *session, char *number) {
+  char *temp = strdup(number);
+
+  session->dial_number_history = g_list_insert(
+				       session->dial_number_history, temp, 1);
+  session_history_normalize(session);
+}
+
+/*
+ * like session_history_add but _appending_ number to list
+ */
+void session_history_append(struct session_t *session, char *number) {
+  char *temp = strdup(number);
+
+  session->dial_number_history = g_list_append(
+					  session->dial_number_history, temp);
+  session_history_normalize(session);
+}
