@@ -10,18 +10,24 @@
 #ifndef _ANT_SESSION_H
 #define _ANT_SESSION_H
 
+#include "config.h"
+
 /* regular GNU system includes */
-#include <termios.h>
+#ifdef HAVE_TERMIOS_H
+  #include <termios.h>
+#endif
 #include <time.h>
 
 /* GTK */
 #include <gtk/gtk.h>
 
 enum state_t {
-  STATE_READY,
-  STATE_RINGING,
-  STATE_DIALING,
-  STATE_CONVERSATION,
+  STATE_READY,          /* completely idle */
+  STATE_RINGING,        /* somebody's calling  */
+  STATE_RINGING_QUIET,  /* same as above, audio off (device blocked) */
+  STATE_DIALING,        /* we are dialing out */
+  STATE_CONVERSATION,   /* we are talking */
+  STATE_SERVICE,        /* special mode (init, settings, llcheck, ...) */
 
   STATE_LAST /* dummy to calculate size */
 };
@@ -106,6 +112,8 @@ struct session_t {
   unsigned int dial_number_history_maxlen; /* how many numbers to remember */
   GtkWidget *status_bar; /* the status bar */
   gint phone_context_id; /* a context for the status bar */
+  GtkWidget *audio_warning; /* inside status bar */
+  gint audio_context_id; /* a context for audio_warning */
 
   GtkWidget *llcheck; /* line level check widget inside status bar */
   GtkWidget *llcheck_in; /* input level meter */
@@ -143,14 +151,22 @@ struct session_t {
 
   char* msn;  /* originating msn, allocated memory! */
   char* msns; /* comma-separated list of msns to listen on, allocated memory!*/
+
+  /* some options (useful for options file handling) */
+  int option_save_options;    /* save options on exit */
+  int option_release_devices; /* close sound devices while not needed */
+  int option_show_llcheck;    /* show line level checks in main window  */
+  int option_show_callerid;   /* show callerid part in main window */
 };
 
 extern struct session_t session;
 
-void session_set_state(struct session_t *session, enum state_t state);
+int session_set_state(struct session_t *session, enum state_t state);
 void session_io_handlers_start(struct session_t *session);
 void session_io_handlers_stop(struct session_t *session);
-void session_reset_audio(struct session_t *session);
+int session_reset_audio(struct session_t *session);
+int session_audio_init(struct session_t *session);
+int session_audio_deinit(struct session_t *session);
 
 int session_init(struct session_t *session,
 		 char *audio_device_name_in,
@@ -168,4 +184,6 @@ void gdk_handle_audio_input(gpointer data, gint fd,
 void gtk_handle_pick_up_button(GtkWidget *widget, gpointer data);
 void gtk_handle_hang_up_button(GtkWidget *widget, gpointer data);
 
+void session_history_add(struct session_t *session, char *number);
+void session_history_append(struct session_t *session, char *number);
 #endif /* session.h */
