@@ -7,10 +7,16 @@
  *
  */
 
+#include "config.h"
+
 /* regular GNU system includes */
 #include <stdio.h>
-#include <stdlib.h>
-#include <sys/time.h>
+#ifdef HAVE_STDLIB_H
+  #include <stdlib.h>
+#endif
+#ifdef HAVE_SYS_TIME_H
+  #include <sys/time.h>
+#endif
 #include <time.h>
 
 /* own header files */
